<?php
/* We ensure that the plugin cannot be accessed directly */
defined( 'ABSPATH' ) or die( 'Plugin file cannot be accessed directly.' );

/* Checks if the current plugin is active or not */
if ( ! class_exists( 'UOC_Video_Shortcode_Admin' ) ) {
	
	class UOC_Video_Shortcode_Admin
	{
		protected static $name = 'UOC-Video-Shortcode-Admin';
		
		private static $initiated = false;
        
        private static $dbManager = null;
		
		public static function init() {			
			
			//In order to detect new connection issues, we need to check the connection if database_logging is activated and $dbManager is null (if null, we are going to create a new connection)
			//And we need to do it every-time so this validation has to be outside the initiated function.
			if($dbManager == null && get_option('uoc_video_settings_database_logging') == 'on'){
				
			     self::update_db_manager();

			}			
			
			if ( ! self::$initiated ) {
				self::init_hooks();
			}
		}
		
		private function update_db_manager(){
			self::$dbManager = new UOC_Video_Shortcode_Manage_MongoDB(
				UOC_VIDEO_SHORTCODE__MONGODB_HOST,
				UOC_VIDEO_SHORTCODE__MONGODB_PORT,
				UOC_VIDEO_SHORTCODE__MONGODB_USER,
				UOC_VIDEO_SHORTCODE__MONGODB_PASSWORD,
				UOC_VIDEO_SHORTCODE__MONGODB_DB,
				UOC_VIDEO_SHORTCODE__MONGODB_COLLECTION
			); 				

			//database_logging is active and there is an issue connecting with the database.
			if(!self::$dbManager->check_connection()){
				update_option('uoc_video_settings_database_logging', '');
				add_settings_error( 'uoc_video_settings_oauth2', 'Database-logging-deactivated', 'UOC Video Shortcode: Database logging has been deactivated due to connection issues. Check the database settings.' );
				//Adds an admin error						
				add_action('admin_notices', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_database_error_notice')); 							
				self::$dbManager = null;
			}
		}
        
        /**
         * Initializes WordPress hooks (filters and actions)
         */
		private static function init_hooks() {
			//We set this variable to true in order to only activate init_hooks one time.
			self::$initiated = true;
			
            /* ADMIN PAGE */
            // Hock for admin menu page
            add_action('admin_menu',  array( 'UOC_Video_Shortcode_Admin','register_uoc_video_shortcode_admin_menu_page'));
            
            /* SETTINGS PAGE */
            //Hock for Settings page
            add_action('admin_menu', array( 'UOC_Video_Shortcode_Admin','register_uoc_video_shortcode_settings_page'));
			
            ///Hock for Settings fields
            add_action('admin_init', array( 'UOC_Video_Shortcode_Admin','register_uoc_video_shortcode_settings'));
            
            /* ADMIN STYLES */
            // Hock for admin styles
            add_action( 'admin_enqueue_scripts', array( 'UOC_Video_Shortcode_Admin','register_uoc_video_shortcode_admin_style'));
			
            /* ADMIN JAVASCRIPTS */
            // Hock for the new tinymce button script (head)
            add_action('admin_head', array( 'UOC_Video_Shortcode_Admin','register_uoc_video_shortcode_admin_tinymce_button')); 	
		
			/* AJAX CALLBACK */
			/* From Wordpress documentation: Both front-end and back-end Ajax requests use admin-ajax.php so is_admin() will always return true */
            /* Wordpress Quest users */
            add_action( 'wp_ajax_nopriv_uoc_video_add_event', array('UOC_Video_Shortcode_Admin','uoc_video_add_event') );
            /* Wordpress Authenticated users */
            add_action('wp_ajax_uoc_video_add_event', array('UOC_Video_Shortcode_Admin','uoc_video_add_event'));			
         }
		
        
		/**
         * Defines the actions to be made when activating the plugin
         */
        public static function activate()
        {             
        } 
    
        
        /**
         * Defines the actions to be made when deactivating the plugin
         */    
        public static function deactivate()
        {
        } 
		
		/**
         * Method to receive the AJAX calls and insert a $document to the database.
         */
        function uoc_video_add_event() {
            
			
			//We check if the current dbManager is active (not null) and if the option to save settings to the database is active
			if(!is_null(self::$dbManager) && get_option('uoc_video_settings_database_logging') == 'on' ) {			
            	//We verify that the WP nonce is valid
				check_ajax_referer( 'uoc_video_add_event' );            

				//We check the minimum $post params
				if(!empty($_POST["video_id"]) && !empty($_POST["video_action"]) && !empty($_POST["video_hash"])){
	
					//video_hash check to validate ajax call values
					if($_POST["video_hash"] == md5(($_POST["video_id"] . $_POST["video_action"] . $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_CURRENT_USER'] . UOC_VIDEO_SHORTCODE__AJAX_CALLS_SALT))) {
					
						//We create the document in an array format
						$document = (new UOC_Video_Shortcode_Event(
							$_SESSION['UOC_VIDEO_SHORTCODE_SESSION_CURRENT_USER'], 
							$_SESSION['UOC_VIDEO_SHORTCODE_SESSION_CURRENT_USER_NICKNAME'],
							$_POST["video_id"],
							$_POST["video_action"],
							$_POST["video_params"]))->toArrayDocument();
						
						try{
							//We save the document 
							self::$dbManager->create($document);
						}catch(Exception $e){
							wp_die(); 							
						}						
					}						
				}
			}            
	        wp_die(); 
		}
		
		/*
		* Shows a database error on the admin side
		*/
		function uoc_video_shortcode_database_error_notice() {
			$class = "error";
			$message = "UOC Video Shortcode: Database logging has been deactivated due to connection issues. Check the database settings.";
			echo"<div class=\"$class\"> <p>$message</p></div>"; 
		}
        
        
		/**
         * Registers and adds the Admin Styles
         */
        function register_uoc_video_shortcode_admin_style() {
            wp_register_style( 'uoc_video_shortcode_admin_css', UOC_VIDEO_SHORTCODE__PLUGIN_URL . 'admin/css/styles.css' );
            wp_enqueue_style( 'uoc_video_shortcode_admin_css' );
        }
        
		/**
         * Registers the filters used to add the shortcode button when rich editor is enabled
         */
        public static function register_uoc_video_shortcode_admin_tinymce_button(){
            if ( get_user_option('rich_editing') == 'true') {
                add_filter('mce_external_plugins', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_add_tinymce_plugin_js'));
                add_filter('mce_buttons', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_add_tinymce_plugin_button'));
            }
        }
        
        
		/**
         * Registers the filters used to add the shortcode buttons
         */
        public static function uoc_video_shortcode_add_tinymce_plugin_js($plugin_array){    
            $plugin_array['uocVSB'] = UOC_VIDEO_SHORTCODE__PLUGIN_URL . 'admin/js/tinymce-plugin.js';
            return $plugin_array;            
        }
        
        
        /**
         * Adds the shortcode button 'uocVSB' to the list of buttons avaiable'
         */
        public static function uoc_video_shortcode_add_tinymce_plugin_button($buttons){
            array_push($buttons, 'uocVSB');
            return $buttons;
        }
        
        
        /**
         * Adds a menu admin page for the plugin
         */
        public static function register_uoc_video_shortcode_admin_menu_page(){
			//uocvideoadminpage has replaced custompage
            add_menu_page( 'UOC Video Shortcode admin page', 'UOC Video Shortcode', 'manage_options', 'uoc_video_admin_page', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_admin_page'), '', 99 ); 
        }
        
        
        /**
         * Adds a settings page for the plugin
         */
        public static function register_uoc_video_shortcode_settings_page() {
            add_options_page('UOC Video Shortcode Settings', 'UOC Video Shortcode', 'manage_options', 'plugin', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_settings_page'));            
        }
        
        
        /**
         * Registers the settings used by the plugin
         */
        public static function register_uoc_video_shortcode_settings() {
            /* https://codex.wordpress.org/Function_Reference/register_setting */
            //register_setting( 'uoc_video_shortcode_settings_options', 'uoc_video_shortcode_settings_name', 'uoc_video_shortcode_settings_validate' );
            register_setting( 'uoc_video_shortcode_settings_options', 'uoc_video_settings_oauth2', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_settings_fields_oauth2_modified_callback'));
            register_setting( 'uoc_video_shortcode_settings_options', 'uoc_video_settings_database_logging', array( 'UOC_Video_Shortcode_Admin','uoc_video_settings_database_logging_modified_callback'));
            
            //We add the general section
            add_settings_section('uoc_video_settings_general', 'Main Settings', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_settings_general_text'), 'uoc_video_settings_page');
            add_settings_field('uoc_video_settings_oauth2', 'Oauth 2 authentication', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_settings_fields_oauth2'), 'uoc_video_settings_page', 'uoc_video_settings_general'); 
            
            add_settings_field('uoc_video_settings_database_logging', 'Database storing', array( 'UOC_Video_Shortcode_Admin','uoc_video_shortcode_settings_fields_database'), 'uoc_video_settings_page', 'uoc_video_settings_general');
        }

        
        /**
         * Renders the Oauth2 settings field
         */
        function uoc_video_shortcode_settings_fields_oauth2() {
            $option = get_option('uoc_video_settings_oauth2');
            $return = '<input name="uoc_video_settings_oauth2" type="checkbox"'. (($option == 'on') ? 'checked' : '') .' />';
            echo $return;
        }
		
		/**
         * Callback to check OAuth 2 settings values. It disables database logging if OAuth2 is deactivated 
         */
		function uoc_video_shortcode_settings_fields_oauth2_modified_callback($input) {
			
			/* If OAuth 2 Option is disabled. Database will be disabled too. */
			if( $input != 'on' ){
				update_option('uoc_video_settings_database_logging', '');
				add_settings_error( 'uoc_video_settings_oauth2', 'Database-logging-deactivated', 'UOC Video Shortcode: Database logging has been deactivated due to OAuth 2 deactivation.' );				
			}

			return $input;		
		}
        
        
        /**
         * Renders the Database logging settings field
         */
        function uoc_video_shortcode_settings_fields_database() {
            $option = get_option('uoc_video_settings_database_logging');
            $return = '<input name="uoc_video_settings_database_logging" type="checkbox"'. (($option == 'on') ? 'checked' : '') .' />';
            echo $return;
        }
		
		
		function uoc_video_settings_database_logging_modified_callback($input){
			//uoc_video_settings_database_logging_modified_callback
			
			if( get_option('uoc_video_settings_oauth2') == 'on' ){
			
				/* We need to check the Database connection if we are going to activate database logging */
				if( $input == 'on' ){
					
					self::update_db_manager();
					
					//If self::$dbManager is null, the connection has failed so we need to disable the logging.
					if(is_null(self::$dbManager)){
						$input = '';					
					}
				}
			
			} else {
				//Restriction: Database logging cannot be activated without OAuth2
				add_settings_error( 'uoc_video_settings_database_logging', 'Database-logging-deactivated', 'Database logging cannot be activated if Oauth2 is deactivated.' );
				$input = '';	
			}

			return $input;			
		}
        
        
        /**
         * Renders the text that will be shown in the 'Main Settings' section of the settings page 
         */
        function uoc_video_shortcode_settings_general_text() {
            echo '<p>Defines if the Oauth 2 Authentication system is activated.</p>';
        }
        
        
        /**
         * Renders the settings page content.
         * Note: We have not used echo methods to avoid render issues with the inputs
         */
        function uoc_video_shortcode_settings_page() {
        ?>
        <div>
            <h1>UOC Video Shortcode Settings</h1>
            Options relating to the Custom Plugin.
            <form action="options.php" method="post">
            <?php settings_fields('uoc_video_shortcode_settings_options'); ?>
            <?php do_settings_sections('uoc_video_settings_page'); ?>         
            <input name="Submit" type="submit" value="<?php esc_attr_e('Save settings'); ?>" />
            </form>
        </div>
        <?php
        }
        
        /**
         * Renders the admin page content.
         */
        public static function uoc_video_shortcode_admin_page(){            
           
            //We create the table
            $uoc_admin_table = new UOC_Video_Shortcode_WP_List_Table();
            
            $uoc_admin_table->prepare_items(self::$dbManager);  
       
            echo '<div class="wrap">';
                 echo '<h2>UOC Video Shortcode Admin page</h2>';
                 echo '<form method="get">';
				 echo '<div class="search-block">';				   
				    echo '<input type="hidden" name="page" value="uoc_video_admin_page">';
                    echo $uoc_admin_table->search_box('search', 'search_field');
				 echo '</div>';
                 echo '<div class="tool-box">';
				//<form method="post">
				//<input type="hidden" name="page" value="admin_search" />
				//<?php $uoc_admin_table->search_box('search', 'search_id');
                //echo $uoc_admin_table->prepare_items($data_array);

                    echo $uoc_admin_table->display(); 
                 echo '</div>';
                echo '</form>';
             echo '</div>';

        }		
	}
}
?>