<?php
/* We ensure that the plugin cannot be accessed directly */
defined( 'ABSPATH' ) or die( 'Plugin file cannot be accessed directly.' );


if ( ! class_exists( 'UOC_Video_Shortcode_Manage_MongoDB' ) ) {
    
    /**
     * Class used to store the $documents to mongoDB
     */
    class UOC_Video_Shortcode_Manage_MongoDB implements UOC_Video_Shortcode_Manage_DataBase {

        private $host;
        private $port;
        private $user;
        private $password;
        private $db;
        private $collection;
        
        
        /**
         * Constructor
         */
		public function __construct($host,$port, $user, $password, $db, $collection)
		{
            $this->host = $host;
            $this->port = $port;
            $this->user = $user;
            $this->password = $password;
            $this->db = $db;
            $this->collection = $collection;
		}
        

        /**
         * Inserts a document to the DB
         */
        public function create($document){
            
            //Gets the collection
            $mongoConnection = $this->connect();
            
            //If collection does not exists, mongodb will reate it
            $collection = $mongoConnection->selectCollection($this->db, $this->collection);
            
            $collection->insert($document);
            $mongoConnection->close();
        }

        /**
         * Gets an array  of documents from the DB
         */
        public function read($documenFilter){
			
			$result = array();
			
			try {            
				//Gets the collection
				$mongoConnection = $this->connect();
				
				//If collection does not exists, mongodb will reate it
				$collection = $mongoConnection->selectCollection($this->db, $this->collection);				
				
				$data_cursor = $collection->find($documenFilter);
				$data_cursor->limit(1000);
				
				//We convert the result to an array
				$result =  iterator_to_array($data_cursor, false);

			} catch(Exception $e){
				$result = array();
			} finally  {					
				$mongoConnection->close();
			}
            
            return $result;  
        }
		
		/**
         * Gets an array of documents from the DB, with possibility to sort, skip and limit the results
         */
        public function read_with_pagination($documenFilter, $sort, $skip, $limit, &$max_items){
			
			$result = array();
            
			try {
				
					//See: MongoDB manual reference operator or ==> $or
				
					//Gets the collection
					$mongoConnection = $this->connect();
					
					//If collection does not exists, MongoDB will create it
					$collection = $mongoConnection->selectCollection($this->db, $this->collection);

					$data_cursor = $collection->find($documenFilter); 

					//We sort the results by date, descendant
					$data_cursor->sort($sort);
					
					//We count all the documents (prior skip&limit)
					$max_items = $data_cursor->count();
					
					//We limit the cursor to the items that will be show on a page (to avoid memory issues using iterator_to_array and to use pagination)
					$data_cursor->skip($skip);
					$data_cursor->limit($limit);
					
					//We convert the result to an array
					$result =  iterator_to_array($data_cursor, false);
					
				}catch(Exception $e){
					$result = array();					
				} finally  {					
					$mongoConnection->close();
				}
            
            
            return $result;  
        }

        /**
         * Updates an action from the DB
         */
        public function update($document){
            //unimplemented method - future work
        }

        /**
         * Deletes an action from the DB
         */
        public function delete($document){      
            //unimplemented method - future work
        } 

        /**
         * Deletes an action from the DB
         */
        public function connect(){   
            try {
                //Gets the connection
                $mongoConnection = new MongoClient('mongodb://'.$this->host.':'.$this->port, $this->get_mongo_options());

            //$mongoConnection->close();
            } catch (MongoConnectionException $e) {
                //Debug Only
				//die('Error connecting to MongoDB server: ' . $e->getMessage());
				die('Unknown error: error number M0001');
            } catch (MongoException $e) {
				//Debug Only
                //die('Error: ' . $e->getMessage());
				die('Unknown error: error number M0002');
            }

            return $mongoConnection;
        }		
		
		public function check_connection(){
			$validConnection = false;
			
			try {
                $mongoConnection = new MongoClient('mongodb://'.$this->host.':'.$this->port, $this->get_mongo_options());
				$mongoConnection->close();	
				$validConnection = true;			
            } catch (Exception $e) {
				$validConnection = false;
            } 
			
			return $validConnection;
		}
		
		protected function get_mongo_options(){
			
			$mongoOptions = array(
				'username' => $this->user,
				'password' => $this->password,
				'db' => $this->db,
			);
			 
			return $mongoOptions;
		}
		
    }
    
}

?>