<?php
/* We ensure that the plugin cannot be accessed directly */
defined( 'ABSPATH' ) or die( 'Plugin file cannot be accessed directly.' );

if ( ! class_exists( 'UOC_Video_Shortcode_WP_List_Table' ) ) {
    
    class UOC_Video_Shortcode_WP_List_Table extends WP_List_Table {
        
        private $maxItems = 0;
        
        public function get_columns(){
            $cols = array('video_id' => 'Video ID',
                'timestamp'    => 'Date',
                'user_name'      => 'User name',
                'user_id'      => 'User ID',
                'action'      => 'Action',
                'parameters'      => 'Parameters'
                );
            
            return $cols;
        }

        /*
		* Function that needs to be overwritten.
		* Prepares de items to be shown on the WP_List_Table
		*/
        public function prepare_items($dbManager) {
            //We get the current page num.
            $currentPage = $this->get_pagenum();
            
            //Search field
            //Sanitized using: https://codex.wordpress.org/Function_Reference/sanitize_text_field
            if(!empty($_GET['s'])){
                $documentFilter = array( '$or' =>
                    array( 
                        array('video_id' => sanitize_text_field($_GET['s'])), 
                        array('user_name' => sanitize_text_field($_GET['s'])), 
                        array('user_id' => sanitize_text_field($_GET['s'])),
                        array('action' => sanitize_text_field($_GET['s'])), 
                        array('parameters' => sanitize_text_field($_GET['s']))
                    )
                );
                
                // Avoids issues with pagination and search
                $currentPage = 1;
            }else{
                $documentFilter = array();               
            }
            
            //Order
            //&orderby=timestamp&order=desc
            if(!empty($_GET['orderby'])){
                //Little security measure to validate Get value
                switch ($_GET['orderby']) {
                    case 'video_id':
                        $documentOrderBy = 'video_id';
                        break;
                    case 'timestamp':
                        $documentOrderBy = 'timestamp';
                        break;
                    case 'user_name':
                        $documentOrderBy = 'user_name';
                        break;
                    case 'action':
                        $documentOrderBy = 'action';
                        break;
                    case 'parameters':
                        $documentOrderBy = 'parameters';
                        break;
                    default: //Default value
                       $documentOrderBy = 'timestamp';
                }                   
            }else{ //Default orderBy field (it has true value in get_sortable_columns)            
                $documentOrderBy = 'timestamp';
            }
            
            if(!empty($_GET['order'])){
                //Little security measure to validate Get value
                switch ($_GET['order']) {
                    case 'asc':
                        $documentOrder = 1;
                        break;
                    case 'desc':
                        $documentOrder = -1;
                        break;
                    default: //Default value
                       $documentOrder = -1;
                }                   
            }else{ //Default order field (it has true value in get_sortable_columns)            
                $documentOrder = -1;
            }
        
            
            
            //We get the items taking into account the current page num.
            $this->items = $this->get_data($dbManager, (($currentPage-1) * 10), $documentFilter, $documentOrderBy, $documentOrder);
            
            
            
            //We get the table columns, hidden columns and sortable columns
            $columns = $this->get_columns();
            $hidden = $this->get_hidden_columns();
            $sortable = $this->get_sortable_columns();

            //We define the pagination
            $this->set_pagination_args( 
                array(
                    'total_items' => $this->maxItems,                  
                    'per_page'    => 10
                )
            );
            
            $this->_column_headers = array($columns, $hidden, $sortable);            
        }
        
        /*
		* Function to get the sortable columns of the table
		*/
        public function get_sortable_columns(){
            $cols = array(
                'video_id' => array('video_id', false),
                'timestamp'    => array('timestamp', true),
                'user_name'      => array('user_name', false),
                'user_id'      => array('user_id', false),
                'action'      =>array('action', false),
                'parameters'      => array('parameters', false)
            );
            
            return $cols;
        }
        
        /*
		* Function to get the hidden columns of the table
		*/
        public function get_hidden_columns(){
            $cols = array();
            
            return $cols;
        }
        
		/*
		* Function to specify what to return for each table column
		*/
        public function column_default( $item, $column_name ) {
            switch( $column_name ) { 
                case 'video_id':                
                case 'user_name':
                case 'user_id':
                case 'action':
                case 'parameters':
                    return $item[ $column_name ];
                case 'timestamp':  
                    date_default_timezone_set(UOC_VIDEO_SHORTCODE__DATE_TIME_ZONE);
                    $date = new DateTime();
                    $date->setTimestamp($item[$column_name]);
                    $date->setTimezone(new DateTimeZone(UOC_VIDEO_SHORTCODE__DATE_TIME_ZONE));
                    return $date->format('d/m/Y  H:i:s');
            }
        }
        
        /*
		* Function to get the data from a dbManager (MongoDB)
		*/
        public function get_data($dbManager, $skip, $documentFilter, $documentOrderBy, $documentOrder){
			
			$result = array();
			
			if(!is_null($dbManager)){
				$sort = array($documentOrderBy => $documentOrder);
				$limit = 10;
                $max_items = 0;
                //Attention: Max items is a by ref variable
				$result = $dbManager->read_with_pagination($documentFilter, $sort, $skip, $limit, $max_items);
                $this->maxItems = $max_items;
			}

			return $result;
        }        
    }
}
?>