<?php

/* We ensure that the plugin cannot be accessed directly */
defined( 'ABSPATH' ) or die( 'Plugin file cannot be accessed directly.' );

/* Checks if the current plugin is active or not */
if ( ! class_exists( 'UOC_Video_Shortcode' ) ) {  
	
	class UOC_Video_Shortcode {
		protected static $name = 'UOC-Video-Shortcode';
		
		private static $initiated = false;
		
		public static function init() {
			if ( ! self::$initiated ) {
				self::init_hooks();
			}
		}
		
		/**
         * Initializes WordPress hooks (filters and actions)
         */
		private static function init_hooks() {
			//We set this variable to true in order to only activate init_hooks one time.
			self::$initiated = true;
            
            /* SCRIPTS */
            // Hock for adding scripts
            add_action( 'wp_enqueue_scripts', array( 'UOC_Video_Shortcode','register_uoc_video_shortcode_scripts'));
            
            /* STYLES */
            // Hock for styles
            add_action( 'wp_enqueue_scripts', array( 'UOC_Video_Shortcode','register_uoc_video_shortcode_style'));
            
            /* SHORTCODE DETECTION + OAUTH 2*/
            // Hock for detecting the shortcode use and use the Oauth 2 authentication if enabled (We must use this hook or headers will already be sent)
            add_action('template_redirect', array( 'UOC_Video_Shortcode', 'register_uoc_video_detect_shortcode' ));
            
            /* SHORTCODE CREATION */
            add_shortcode('UOC-VIDEO', array('UOC_Video_Shortcode','register_uoc_video_shortcode'));
		}
        
        
        /**
         * Registers and adds the Public Styles (popcorn.js)
         */
        function register_uoc_video_shortcode_scripts() {
            //we make sure jquery is included (From wordpress)
            wp_enqueue_script( 'jquery' );
            wp_enqueue_script( 'popcorn-complete', UOC_VIDEO_SHORTCODE__PLUGIN_URL . 'public/js/popcorn-complete.js', array(), false, false );
        }
        
        /**
         * Registers and adds the Styles
         */
        function register_uoc_video_shortcode_style() {
            wp_register_style( 'uoc_video_shortcode_css', UOC_VIDEO_SHORTCODE__PLUGIN_URL . 'public/css/styles.css' );
            wp_enqueue_style( 'uoc_video_shortcode_css' );
        }
		
		/**
         * Translates the shortcode into the real html/javascript code
         */  
        public static function register_uoc_video_shortcode($atts){
		/* Shortcodes atributes */
		/* Default values:  https://codex.wordpress.org/Function_Reference/shortcode_atts */
            $a = shortcode_atts( array(
                'id' => '00000',
                'type' => 'vimeo',
                'url' => 'https://player.vimeo.com/video/114014468',
                'title_type' => '',
                'title_text' => 'Placeholder title',
                'links' => '',
                'video_width' => '600',
                'video_height' => '400',
            ), $atts );


            $links = explode('||;;||', $a['links']);            
            
            $video_js = [];
            
            //VideoID Sanitize (If video ID is not a valid ID, all will fail)
            $video_id = preg_replace("/[^a-zA-Z0-9\s]/", "", $a['id']);
            $video_id = preg_replace('/\s/', '_', $video_id);
            
            
            $video_container_var = 'uoc_video_container'. $video_id;
            // Start video script tag
            array_push($video_js, '<script>');
            //Start js script
            array_push($video_js, 'jQuery(document).ready(function($) {');  
            
            //http://popcornjs.org/popcorn-docs/media-wrappers/
            //Note: youtube type is unstable but otherwise it seems to work well.
            //We will not add any wizard for this wrapper but can be used if added manually
            if($a['type'] == 'youtube'){
                array_push($video_js, 'var uoc_video_wrapper = Popcorn.HTMLYouTubeVideoElement(\'#uoc_video_'.$video_id.'\');');
                array_push($video_js, 'uoc_video_wrapper.src = \''.$a['url'].'?controls=2&origin='.UOC_VIDEO_SHORTCODE__PLUGIN_URL.'\';'); 
			//vimeo
            }else{                
                array_push($video_js, 'var uoc_video_wrapper = Popcorn.HTMLVimeoVideoElement(\'#uoc_video_'.$video_id.'\');');
                array_push($video_js, 'uoc_video_wrapper.src =\''.$a['url'].'\';'); 
            }
            
            array_push($video_js, 'var '. $video_container_var .' = Popcorn( uoc_video_wrapper );'); 
            
            
            $result = [];
            //Start UOC Video shortcode block
            array_push($result, '<div class="uoc_video_shortcode">');
            
            //Title
            if($a['title_type'] != ''){
                array_push($result, '<'.$a['title_type'].'>'.$a['title_text'].'</'.$a['title_type'].'>');
            }            
            	
			//Video player block
			array_push($result, '<div id="uoc_video_'.$video_id.'" style="width:'.$a['video_width'].'px;height:'.$a['video_height'].'px"></div>');
			
			
			/* Event logging to database */
			if (get_option('uoc_video_settings_database_logging', '') == 'on'){
				
                //Page reloaded
                array_push($video_js, 'if(sessionStorage.uoc_video && sessionStorage.uoc_video === window.location.href){');
                    array_push($video_js, self::generate_uoc_video_shortode_javascript($video_id, 'page_reload', ''));
                //New page/ or different tab reloaded
                array_push($video_js, '}else{');
                    array_push($video_js, 'sessionStorage.uoc_video = window.location.href;');
                    array_push($video_js, self::generate_uoc_video_shortode_javascript($video_id, 'pageload', ''));
                array_push($video_js, '}');
				
				
				//Page unload
				array_push($video_js, '$(window).bind(\'beforeunload\', function(){');
				array_push($video_js, self::generate_uoc_video_shortode_javascript($video_id, 'page_unload_or_refresh', ''));
				array_push($video_js, '});');				
				
				// Listen for pause event
				array_push($video_js, $video_container_var.'.on(\'pause\', function( e ) {');
				array_push($video_js, self::generate_uoc_video_shortode_javascript($video_id, 'pause', '\' + ' . $video_container_var . '.currentTime() + \''));
				array_push($video_js, '});');
				
				// Listen for play event
				array_push($video_js, $video_container_var.'.on(\'play\', function( e ) {');
				array_push($video_js, self::generate_uoc_video_shortode_javascript($video_id, 'play', '\' + this.currentTime() + \''));
				array_push($video_js, '});');
				
				//seeking or seeked
                array_push($video_js, $video_container_var.'.on(\'seeked\', function( e ) {');
				array_push($video_js, self::generate_uoc_video_shortode_javascript($video_id, 'seeked', '\' + this.currentTime() + \''));
				array_push($video_js, '});');
				
            }
			  
            //UOC Video shortcode links
            if(count($links)>0){
                array_push($result, '<ul>');
                foreach ($links as $clave => $valor) {
                    $link =  explode('||,,||', $valor);
                    if(count($link)==2){
                        $linkId = str_replace(":", "", $link[1]);
                        array_push($result, '<li id="uoc_video_'.$video_id.'link'.$linkId.'"><a>'. $link[0].'</a></li>');                        
                        
						//click link behaviour
                        array_push($video_js, '$(\'#uoc_video_'.$video_id.'link'.$linkId.'\').click(function(){');
                        array_push($video_js, $video_container_var .'.currentTime('.trim($link[1]).');');
                        array_push($video_js, $video_container_var .'.play();');                       
 
						//Only if the database logging option is active, we generate the corresponding js ajax calls
						if (get_option('uoc_video_settings_database_logging', '') == 'on'){
							array_push($video_js, self::generate_uoc_video_shortode_javascript($video_id, 'clickindex', trim($link[1])));
						}
                          
						//Debug ajax response code (usually disabled):
                        //array_push($video_js, ', function(response) {');
                        //array_push($video_js, 'alert(response);');
                        //array_push($video_js, '});');       
                        //array_push($video_js, ');');
                        
                        
                        array_push($video_js, '})');                         
                        
                    }else{
                        array_push($result, '<li>Link error. Check shortcode</li>');                        
                    }
                }
                array_push($result, '</ul>');
            }
            
            //End jquery script
            array_push($video_js, '});'); 
            // End video script tag
            array_push($video_js, '</script>');
            array_push($result, implode("\n", $video_js));
            //End UOC Video shortcode block
            array_push($result, '</div>');
            
            return implode("\n", $result);		
	    }
		
		private function generate_uoc_video_shortode_javascript($video_id, $action, $params){
			
			$js= [];			

			array_push($js, '$.post(\''. admin_url('admin-ajax.php'). '\', {'); 
			array_push($js, 'action: \'uoc_video_add_event\','); 
			array_push($js, 'video_id: \''.$video_id.'\','); 
			array_push($js, 'video_action: \''.$action.'\','); 
			array_push($js, 'video_params: \''.$params.'\','); 
            
            //Params cannot be hased as they are generated on client side.
			array_push($js, 'video_hash: \''.md5(((string) $video_id) . ((string) $action) . ((string) $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_CURRENT_USER']) . ((string) UOC_VIDEO_SHORTCODE__AJAX_CALLS_SALT)).'\','); 
			//_wpnonce: WP Security measure: https://codex.wordpress.org/WordPress_Nonces
			array_push($js, '_wpnonce: \''.wp_create_nonce('uoc_video_add_event').'\','); 
			array_push($js, '});');			
			
			return implode("\n", $js);
			
		}
        
        
        /**
         * Translates the shortcode into the real html/javascript code
         */ 
        public static function register_uoc_video_detect_shortcode() {
            
            //Protocol used
            $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' 
                || $_SERVER['SERVER_PORT'] == 443) ? 'https://' : 'http://';

            //Keeps the wordpress page we are visiting
            if(!empty($_GET['p'])){                            
                $params = '?p='.$_GET['p']; 
            }else{
                $params = '';    
            }
            
            $redirectUri = $protocol.$_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'].$params;
            
            
			
			//We ensure that the user has not been authenticated forever in order to avoid possible issues with public pcs if by some reason
			//an user do not close the browser or decides to save the current session
			if(isset($_SESSION['UOC_VIDEO_SHORTCODE_SESSION_AUTH_TIME'])){
				if(time() - $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_AUTH_TIME']  >= UOC_VIDEO_SHORTCODE__MAX_AUTH_SESSION_TIME_SECONDS){
					session_unset(); 
                    header('Location: '.$redirectUri);
					die();					
				}				
			}
            
            
            //Checks if the user is authenticated or if Oauth 2 authentication is disabled
            if(!isset($_SESSION['UOC_VIDEO_SHORTCODE_SESSION_CURRENT_USER']) && (get_option('uoc_video_settings_oauth2', '') == 'on')){            
                //Global post variable used to detect the shortcode
                global $post;
                //We define a pattern that will be used to search for shortcodes
                $pattern = get_shortcode_regex();
                if (!empty($post->post_content)) {
                    //https://codex.wordpress.org/Function_Reference/get_shortcode_regex
                    //If we found the shortcode
                    if (   preg_match_all( '/'. $pattern .'/s', $post->post_content, $matches )
                        && array_key_exists( 2, $matches )
                        && in_array( 'UOC-VIDEO', $matches[2] ) )
                    {
                        //Provider (UOC)
                        $provider = new League\OAuth2\Client\Provider\UOC([
                            'clientId'      => UOC_VIDEO_SHORTCODE__OAUTH2_CLIENT_ID,
                            'clientSecret'  => UOC_VIDEO_SHORTCODE__OAUTH2_CLIENT_SECRET,
                            'redirectUri'   => $redirectUri,
                            'scopes'        => ['READ'],
                        ]);

                        //We need the code from the Authorization Server.
                        //We save the state for further security checks
                        if (!isset($_GET['code'])) {
                            $authUrl = $provider->getAuthorizationUrl();
                            $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_OAUTH2_STATE'] = $provider->state;
                            session_write_close();
                            header('Location: '.$authUrl);
                            exit;

                        // Checks if the stored state is the same of the returned one to avoid CSRF attack
                        } elseif (empty($_GET['state']) || ($_GET['state'] !== $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_OAUTH2_STATE'])) {
                            unset($_SESSION['UOC_VIDEO_SHORTCODE_SESSION_OAUTH2_STATE']);
                            session_unset(); 
                            header('Location: '.$redirectUri);
                            die();
                        } else {

                            try {                                
                                // We get the $token
                                $token = $provider->getAccessToken('authorization_code', [
                                    'code' => $_GET['code']
                                ]); 

                                // We get the authenticated user details                           
                                $authenticatedUser = $provider->getUserDetails($token);
                                
                                // We save the authenticated user uid for further uses
                                $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_CURRENT_USER'] = $authenticatedUser->uid;
                                $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_CURRENT_USER_NICKNAME'] = $authenticatedUser->nickname;
								//We save the moment the user authenticates, in order to
								$_SESSION['UOC_VIDEO_SHORTCODE_SESSION_AUTH_TIME'] = time();

                            } catch (Exception $e) {
                                print 'Unexpected error occured';
                                exit();
                            }
                        }
                    }
                }
            }            
        }
    }
}
?>