<?php

/*
Plugin Name: UOC Video Shortcode
Plugin URI:  http://www.uoc.edu
Description: Plugin created for Open University Of Catalonia (UOC) to allow the easy creation of videos using  Wordpress shortcodes.
Version:     1.0
Author:      japeiro@uoc.edu
Author URI:  http://www.uoc.edu
License:     BSD-3 Clause
License URI: http://opensource.org/licenses/BSD-3-Clause


Copyright(c) 2015, Josep Peiró Bonet (japeiro@uoc.edu), Universitat Oberta de Catalunya

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


/* Security */
/* We ensure that the plugin cannot be accessed directly */
defined( 'ABSPATH' ) or die( 'Plugin file cannot be accessed directly.' );

//General values
define( 'UOC_VIDEO_SHORTCODE__PLUGIN_URL', plugin_dir_url( __FILE__ ) );
define( 'UOC_VIDEO_SHORTCODE__PLUGIN_DIR', plugin_dir_path( __FILE__ ) );

//Settings values
require_once( UOC_VIDEO_SHORTCODE__PLUGIN_DIR . 'admin/settings/settings.php' );

//SESSION LOADING
//We need to make sure that the session has started http://php.net/manual/es/function.session-start.php
//as Oauth 2 can't work without sessions (to avoid CSRF attack)
//https://www.owasp.org/index.php/PHP_Security_Cheat_Sheet#Authentication_and_Session_Management_Cheat_Sheet
add_action('init', 'uoc_video_shortcode_start_session', 1);

function uoc_video_shortcode_start_session() {    
    
    //We need to check that a session is started
    if(!session_id()) {
        session_start();
    }
    
    //Anti session Fixation measure
    if (!isset($_SESSION['UOC_VIDEO_SHORTCODE_SESSION_STARTED']))
    {
        session_regenerate_id(true);
        $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_STARTED'] = true;
    }

    //Anti session hijaking measures (best-effort)
    if (isset($_SESSION['UOC_VIDEO_SHORTCODE_SESSION_HTTP_USER_AGENT']) && isset($_SESSION['UOC_VIDEO_SHORTCODE_SESSION_REMOTE_ADDR']))
    {
        if ($_SESSION['UOC_VIDEO_SHORTCODE_SESSION_HTTP_USER_AGENT'] != md5($_SERVER['HTTP_USER_AGENT'].UOC_VIDEO_SHORTCODE__SESSION_USER_AGENT_SALT) || 
                $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_REMOTE_ADDR'] != md5($_SERVER['REMOTE_ADDR'].UOC_VIDEO_SHORTCODE__SESSION_REMOTE_ADDR_SALT))
        {
            session_unset();     
            header("Refresh:0");
            exit;
        }
    }
    else
    {
        $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_HTTP_USER_AGENT'] = md5($_SERVER['HTTP_USER_AGENT'] .UOC_VIDEO_SHORTCODE__SESSION_USER_AGENT_SALT);
        $_SESSION['UOC_VIDEO_SHORTCODE_SESSION_REMOTE_ADDR'] = md5($_SERVER['REMOTE_ADDR'] .UOC_VIDEO_SHORTCODE__SESSION_REMOTE_ADDR_SALT);
    }    
}

/* OAUTH 2 Client COMPONENT AUTO-LOAD */
require UOC_VIDEO_SHORTCODE__PLUGIN_DIR . 'includes/vendor/autoload.php';

/* SHORTCODE-PLUGIN PUBLIC FILES */
require_once( UOC_VIDEO_SHORTCODE__PLUGIN_DIR . 'public/class-uoc-video-shortcode.php' );
add_action( 'init', array( 'UOC_Video_Shortcode', 'init' ) );   


/* SHORTCODE-PLUGIN ADMIN FILES */
if ( is_admin() ) {
	/* Database Storage & MongoDB required files */
	require_once( UOC_VIDEO_SHORTCODE__PLUGIN_DIR . 'admin/events/class-uoc-video-shortcode-event.php' ); 
	require_once( UOC_VIDEO_SHORTCODE__PLUGIN_DIR . 'admin/database/interface-uoc-video-shortcode-manage-database.php' );    
	require_once( UOC_VIDEO_SHORTCODE__PLUGIN_DIR . 'admin/database/class-uoc-video-shortcode-manage-mongodb.php' ); 
	
	/* We check that Admin table (WP_List_Table) is loaded as UOC_Video_Shortcode_WP_List_Table extends from WP_List_Table  */
	if(!class_exists( 'WP_List_Table' ) ) {
		require_once(ABSPATH.'wp-admin/includes/class-wp-list-table.php' );
	}
	require_once( UOC_VIDEO_SHORTCODE__PLUGIN_DIR . 'admin/tables/class-uoc-video-shortcode-wp-list-table.php' );   

    /* UOC Video Shortcode admin side */
	require_once( UOC_VIDEO_SHORTCODE__PLUGIN_DIR . 'admin/class-uoc-video-shortcode-admin.php' ); 
	add_action( 'init', array( 'UOC_Video_Shortcode_Admin', 'init' ) );       
}

?>